from splent_framework.seeders.BaseSeeder import BaseSeeder


class ResetSeeder(BaseSeeder):
    def run(self):
        data = [
            # Create any Model object you want to make seed
        ]

        self.seed(data)



from splent_framework.db import db


class ResetToken(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    token = db.Column(db.String(256), unique=True, nullable=False)
    used_at = db.Column(db.DateTime, nullable=True)

    def __repr__(self):
        return f"Reset<{self.id}>"



from datetime import datetime, timezone

from flask import abort, current_app, url_for
from itsdangerous import BadTimeSignature, SignatureExpired, URLSafeTimedSerializer
from werkzeug.security import generate_password_hash

from splent_framework.db import db
from splent_framework.services.BaseService import BaseService
from splent_io.splent_feature_auth.models import User
from splent_io.splent_feature_reset.models import ResetToken
from splent_io.splent_feature_reset.repositories import ResetRepository

TOKEN_MAX_AGE = 3600  # 1 hour


class ResetService(BaseService):
    def __init__(self):
        super().__init__(ResetRepository())

    def get_serializer(self):
        return URLSafeTimedSerializer(current_app.config["SECRET_KEY"])

    def send_reset_password_mail(self, email: str) -> str | None:
        user = User.query.filter_by(email=email).first()
        if not user:
            return None

        s = self.get_serializer()
        token = s.dumps(email, salt="email-confirm")
        link = url_for("reset.reset_password", token=token, _external=True)
        body = f"Your link to reset your password is {link}"

        mail_service = current_app.extensions.get("splent_mail_service")
        if not mail_service:
            raise RuntimeError(
                "Mail service not available. Is splent_feature_mail installed?"
            )
        mail_service.send_email("Password Reset Request", [email], body)

        return token

    def add_token(self, token: str | None):
        if token is None:
            return
        reset_token = ResetToken(token=token)
        db.session.add(reset_token)
        db.session.commit()

    def get_email_by_token(self, token: str) -> str:
        s = self.get_serializer()
        return s.loads(token, salt="email-confirm", max_age=TOKEN_MAX_AGE)

    def check_valid_token(self, token: str):
        s = self.get_serializer()
        try:
            s.loads(token, salt="email-confirm", max_age=TOKEN_MAX_AGE)
        except (SignatureExpired, BadTimeSignature):
            abort(404)

    def token_already_used(self, token: str) -> bool:
        reset_token = self.repository.get_by_column("token", token)
        if not reset_token:
            return False
        return reset_token[0].used_at is not None

    def reset_password(self, email: str, password: str):
        user = User.query.filter_by(email=email).first()
        if not user:
            abort(404)
        user.password = generate_password_hash(password)
        db.session.commit()

    def mark_token_as_used(self, token: str):
        reset_token = self.repository.get_by_column("token", token)
        if reset_token:
            reset_token[0].used_at = datetime.now(timezone.utc)
            db.session.commit()



"""Alembic migration environment for splent_feature_reset."""

from splent_io.splent_feature_reset import models  # noqa: F401
from splent_framework.migrations.feature_env import run_feature_migrations

FEATURE_NAME = "splent_feature_reset"
FEATURE_TABLES = {"reset_token"}

run_feature_migrations(FEATURE_NAME, FEATURE_TABLES)



from splent_io.splent_feature_reset.models import ResetToken
from splent_framework.repositories.BaseRepository import BaseRepository


class ResetRepository(BaseRepository):
    def __init__(self):
        super().__init__(ResetToken)



from splent_framework.hooks.template_hooks import register_template_hook
from flask import render_template, url_for


def inject_reset_link():
    return render_template("hooks/reset_link.html")


register_template_hook("auth.login.form_footer", inject_reset_link)
register_template_hook("auth.signup.form_footer", inject_reset_link)


# ── Script hooks ─────────────────────────────────────────────────────────────


def reset_scripts():
    return (
        '<script src="'
        + url_for("reset.assets", subfolder="dist", filename="reset.bundle.js")
        + '"></script>'
    )


register_template_hook("layout.scripts", reset_scripts)



from flask import flash, redirect, render_template, request, url_for
from splent_io.splent_feature_auth.decorators import guest_required
from splent_io.splent_feature_reset import reset_bp
from splent_io.splent_feature_reset.services import ResetService

reset_service = ResetService()


@reset_bp.route("/reset/forgot", methods=["GET", "POST"])
@guest_required
def forgot():
    if request.method == "POST":
        email = request.form["email"]
        try:
            token = reset_service.send_reset_password_mail(email=email)
            reset_service.add_token(token=token)
            flash(
                "Check your email for the instructions to reset your password",
                "success",
            )
        except Exception as exc:
            flash(
                f"An error occurred while trying to reset the password: {exc}",
                "danger",
            )
        return redirect(url_for("auth.login"))
    return render_template("forgot/forgot.html")


@reset_bp.route("/reset/password/<token>", methods=["GET", "POST"])
@guest_required
def reset_password(token):
    reset_service.check_valid_token(token)
    email = reset_service.get_email_by_token(token)

    if reset_service.token_already_used(token):
        flash("This reset link has already been used.", "danger")
        return redirect(url_for("auth.login"))

    if request.method == "POST":
        password = request.form["password"]
        reset_service.reset_password(email=email, password=password)
        reset_service.mark_token_as_used(token)
        flash("Your password has been updated!", "success")
        return redirect(url_for("auth.login"))
    return render_template("forgot/reset_password.html")



"""splent_feature_reset

Revision ID: 1c2ed70161dd
Revises:
Create Date: 2026-03-21 18:47:37.395421

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "1c2ed70161dd"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "reset_token",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("token", sa.String(length=256), nullable=False),
        sa.Column("used_at", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("token"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("reset_token")
    # ### end Alembic commands ###
